/****************************************************************************/
/* Copyright (C) 2009-2011, Heartland.Data.co.  All Rights Reserved.		*/
/*																			*/
/* Title	: DT10 Driver	[All]											*/
/* Author	: HLDC															*/
/****************************************************************************/

#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <sys/mman.h>
#include <asm/page.h>

/* ========== Parameter ========== */
#define		DT_UINT32				unsigned int	/* Test Point argument type */
#define		DT_VARIABLE_BIT			0x02			/* Memory Dump MASK */
#define		DT_EVTTRG_BIT			0x08			/* EventTrigger */
/* ============================================ */

#define		DT_INT_DISABLE()
#define		DT_INT_RESTORE()

static int fd = -1;
static void *cookie = NULL;

/*===============================================================================
	Func	: _TP_BusOut
	Desc	: Called by Test Point
===============================================================================*/
void _TP_BusOut( DT_UINT32 addr, DT_UINT32 dat )
{
	void	*p ;

	if( fd < 0 )
	{
		fd = open("/dev/mem", O_RDWR);
		if( fd < 0 ) return ;
		cookie = mmap( NULL, PAGE_SIZE, PROT_WRITE | PROT_READ, MAP_SHARED, fd, 0x60000000 );
	}

/*
The Linux has the file access system for memory space: “/dev/mem”; so you should 
mmap to “/dev/mem”.
	
Please modify the 2nd argument PAGE_SIZE and the 6th argument: initial address of 
FlashROM of mmap function matching to your target environment.
*/

	p = cookie + addr ;
	*(unsigned short *)p = dat;

}

/*===============================================================================
	Func	: _TP_MemoryOutput
	Desc	: Called by Variable Test Point
===============================================================================*/
void _TP_MemoryOutput( DT_UINT32 addr, DT_UINT32 dat, void *value, DT_UINT32 size)
{
	DT_UINT32 i;
	unsigned char *p;

	DT_INT_DISABLE();													/* Disable Interrupt */
	_TP_BusOut( (addr | DT_VARIABLE_BIT), dat);							/* Memory Dump Start */
	if (size >= 256) size = 256;
	if(value){
		p = (unsigned char *)value;
		for (i = 0; i < size; i += 2) {
			dat = p[i];
			if ((i + 1) < size) {
				dat |= p[i+1] << 8;
			}
			addr = DT_VARIABLE_BIT;			/* Variable Bit */
			if (i == 0) {
				addr |= size << 8;			/* size */
			}
			_TP_BusOut( addr, dat );
		}
	}
	DT_INT_RESTORE();													/* Enable Interrupt */ 
}

/*===============================================================================
	Func	: _TP_EventTrigger
	Desc	: Called by Event Trigger Test Point
===============================================================================*/
void _TP_EventTrigger( DT_UINT32 addr, DT_UINT32 dat)
{
	_TP_BusOut( addr | DT_EVTTRG_BIT  , ((unsigned short)(dat&0x0FFF))<<4 );
}

